from percentage import percentage
# import percentage.captions

Percentage = percentage
MinMaxLevelNumber = percentage.MinMaxLevelNumber
PercentageCaption = percentage.PercentageCaption

# PercentageValueCaption = captions.PercentageValueCaption
# PercentageTinyTimeSpanCaption = captions.PercentageTinyTimeSpanCaption
